<?php

/**
* @package mod-tracker
* @category mod
* @author Valery Fremaux
* @version MOODLE 2.0
* @date 05/06/2009
*
* Controller for all "view" related views
* 
* @usecase submitanissue
* @usecase updateanissue
* @usecase delete
* @usecase updatelist
* @usecase addcomment (form)
* @usecase doaddcomment
* @usecase usequery
* @usecase register
* @usecase unregister
* @usecase cascade
* @usecase raisepriority
* @usecase lowerpriority
* @usecase raisetotop
* @usecase lowertobottom
* @usecase askraise (form)
* @usecase doaskraise
*/

if (!defined('MOODLE_INTERNAL')) {
    die('Direct access to this script is forbidden.');    ///  It must be included from view.php in mod/tracker
}

/************************************* Submit an issue *****************************/
if ($action == 'submitanissue'){
	if (!$issue = tracker_submitanissue($tracker)){
	   print_error('errorinvalidissue', 'tracker');
    }
    print_simple_box_start('center', '80%', '', '', 'generalbox', 'bugreport');
    print_string('thanks', 'tracker');
    print_simple_box_end();
    print_continue("view.php?id={$cm->id}view=view&amp;screen=browse");
    
    // notify all admins
    if ($tracker->allownotifications){
        tracker_notify_submission($issue, $cm, $tracker);
    }
    
    return -1;
}
/************************************* update an issue *****************************/
elseif ($action == 'updateanissue'){
    $issue->id = required_param('issueid', PARAM_INT);
    $issue->status = required_param('status', PARAM_INT);
    $issue->assignedto = required_param('assignedto', PARAM_INT);
    $issue->summary = required_param('summary', PARAM_TEXT);
    $issue->description = addslashes(required_param('description', PARAM_CLEANHTML));
    $issue->format = required_param('format', PARAM_INT);
    $issue->datereported = required_param('datereported', PARAM_INT);
    $issue->resolution = addslashes(required_param('resolution', PARAM_CLEANHTML));
    $issue->resolutionformat = required_param('resolutionformat', PARAM_INT);
    $issue->trackerid = $tracker->id;

    // if ownership has changed, prepare logging
    $oldrecord = $DB->get_record('tracker_issue', array('id' => $issue->id));
    
    if ($oldrecord->assignedto != $issue->assignedto){
        $ownership->trackerid = $tracker->id;
        $ownership->issueid = $oldrecord->id;
        $ownership->userid = $oldrecord->assignedto;
        $ownership->bywhomid = $oldrecord->bywhomid;
        $ownership->timeassigned = ($oldrecord->timeassigned) ? $oldrecord->timeassigned : time();
        if (!$DB->insert_record('tracker_issueownership', $ownership)){
            print_error ('errorlogownership', 'tracker');
        }        
    }    
    
    $issue->bywhomid = $USER->id;
    $issue->timeassigned = time();

    if (!$DB->update_record('tracker_issue', $issue)){
        print_error ('errorissueupdate');
    }

    // if not CCed, the assignee should be
    tracker_register_cc($tracker, $issue, $issue->assignedto);

    /// send state change notification
    if ($oldrecord->status != $issue->status){
        tracker_notifyccs_changestate($issue->id, $tracker);
    }

    tracker_clearelements($issue->id);    
    tracker_recordelements($issue);
    
    // TODO : process dependancies
    $dependancies = optional_param('dependancies', null, PARAM_INT);
    if (is_array($dependancies)){
        // cleanup previous depdendancies
        if (!$DB->delete_records('tracker_issuedependancy', array('childid' => $issue->id))){
            print_error('errordeletedeps', 'tracker');
        }
        // install back new one
        foreach($dependancies as $dependancy){
            $dependancyrec->trackerid = $tracker->id;
            $dependancyrec->parentid = $dependancy;
            $dependancyrec->childid = $issue->id;
            $dependancyrec->comment = '';
            if (!$DB->insert_record('tracker_issuedependancy', $dependancyrec)){
                print_error('erroradddep', 'tracker');
            }
        }
    }
}
/************************************* delete an issue record *****************************/
elseif ($action == 'delete'){
    $issueid = required_param('issueid', PARAM_INT);

    $maxpriority = $DB->get_field('tracker_issue', 'resolutionpriority', array('id' => $issueid));

    $DB->delete_records('tracker_issue', array('id' => $issueid));
    $DB->delete_records('tracker_issuedependancy', array('childid' => $issueid));
    $DB->delete_records('tracker_issuedependancy', array('parentid' => $issueid));
    $DB->delete_records('tracker_issueattribute', array('issueid' => $issueid));
    $DB->delete_records('tracker_issuecomment', array('issueid' => $issueid));
    $DB->delete_records('tracker_issueownership', array('issueid' => $issueid));

    // lower priority of every issue above
    
    $sql = "
        UPDATE
            {tracker_issue}
        SET
            resolutionpriority = resolutionpriority - 1
        WHERE
            trackerid = ? AND
            resolutionpriority > ?
    ";

    $DB->execute_sql($sql, array($tracker->id, $maxpriority));

    // todo : send notification to all cced

    $DB->delete_records('tracker_issuecc', array('issueid' => $issueid));
}
/************************************* updating list and status *****************************/
elseif ($action == 'updatelist'){
	$keys = array_keys($_POST);							    // get the key value of all the fields submitted
	$statuskeys = preg_grep('/status./' , $keys);  	        // filter out only the status
	$assignedtokeys = preg_grep('/assignedto./' , $keys);  	// filter out only the assigned updating
	$newassignedtokeys = preg_grep('/assignedtoi./' , $keys);  // filter out only the new assigned
	foreach($statuskeys as $akey){
	    $issueid = str_replace('status', '', $akey);
	    $haschanged = optional_param('schanged'.$issueid, 0, PARAM_INT);
	    if ($haschanged){
    	    $issue->id = $issueid;
    	    $issue->status = required_param($akey, PARAM_INT);
    	    $oldstatus = $DB->get_field('tracker_issue', 'status', array('id' => $issue->id));
    	    $DB->update_record('tracker_issue', $issue);    
    	    /// check status changing and send notifications
    	    if ($oldstatus != $issue->status){
        	    if ($tracker->allownotifications){
        	        tracker_notifyccs_changestate($issue->id, $tracker);
        	    }
        	}
        }
	}

	/// always add a record for history
	foreach($assignedtokeys as $akey){
	    $issueid = str_replace('assignedto', '', $akey);
	    // new ownership is triggered only when a change occured
	    $haschanged = optional_param('changed'.$issueid, 0, PARAM_INT);
	    if ($haschanged){
	        // save old assignement in history
            $oldassign = $DB->get_record('tracker_issue', array('id' => $issueid));
            if ($oldassign->assignedto != 0){
                $ownership->trackerid = $tracker->id;
                $ownership->issueid = $issueid;
        	    $ownership->userid = $oldassign->assignedto;
        	    $ownership->bywhomid = $oldassign->bywhomid;
        	    $ownership->timeassigned = $oldassign->timeassigned;
        	    if (!$DB->insert_record('tracker_issueownership', $ownership)){
        	        print_error('erroraddownership', 'tracker', $issueid);
        	    }
        	}

            // update actual assignement
    	    $issue->id = $issueid;
    	    $issue->bywhomid = $USER->id;
    	    $issue->timeassigned = time();
    	    $issue->assignedto = required_param($akey, PARAM_INT);
    	    tracker_register_cc($tracker, $issue, $issue->assignedto);
    	    if (!$DB->update_record('tracker_issue', $issue)){
    	        print_error('errorupdateassignee', 'tracker', $issueid);
    	    }    	    

    	    if ($tracker->allownotifications){
    	        tracker_notifyccs_changeownership($issue->id, $tracker);
    	    }
    	}
	}

	/// reorder priority field and discard newly resolved or abandonned
	tracker_update_priority_stack($tracker);
}
/********************************* requires the add a comment form **************************/
elseif ($action == 'addacomment'){
    $form->issueid = required_param('issueid', PARAM_INT);
    include "views/addacomment.html";
    return -1;
}
/***************************************** add a comment ***********************************/
elseif ($action == 'doaddcomment'){
    $issueid = required_param('issueid', PARAM_INT);
    $comment->comment = addslashes(required_param('comment', PARAM_CLEANHTML));
    $comment->commentformat = required_param('commentformat', PARAM_INT);
    
    $comment->userid = $USER->id;
    $comment->trackerid = $tracker->id;
    $comment->issueid = $issueid;
    $comment->datecreated = time();
    if (!$DB->insert_record('tracker_issuecomment', $comment)){
        print_error('erroraddcomment', 'tracker', $issueid);
    }

    if ($tracker->allownotifications){
        tracker_notifyccs_comment($issueid, $comment->comment, $tracker);
    }
    $issue = $DB->get_record('tracker_issue', array('id' => $issueid));
	tracker_register_cc($tracker, $issue, $USER->id);
}
/************************************ reactivates a stored search *****************************/
elseif($action == 'usequery'){
    $queryid = required_param('queryid', PARAM_INT);
    $fields = tracker_extractsearchparametersfromdb($queryid);
}
/******************************* unregister administratively a user *****************************/
elseif ($action == 'unregister'){
	$issueid = required_param('issueid', PARAM_INT);
	$ccid = required_param('ccid', PARAM_INT);
	if (!$DB->delete_records ('tracker_issuecc', array('trackerid' => $tracker->id, 'issueid' => $issueid, 'userid' => $ccid))){
		print_error('errordeletecc', 'tracker', $issueid);
	}
}
elseif ($action == 'register'){
	$issueid = required_param('issueid', PARAM_INT);
	$ccid = required_param('ccid', PARAM_INT);
	$issue = $DB->get_record('tracker_issue', array('id' => $issueid));
	tracker_register_cc($tracker, $issue, $ccid);
}
/******************************* copy an issue to a parent tracker *****************************/
elseif ($action == 'cascade'){
    global $USER;

	$issueid = required_param('issueid', PARAM_INT);
	$issue = $DB->get_record('tracker_issue', array('id' => $issueid));
	$attributes = $DB->get_records('tracker_issueattribute', array('issueid' => $issue->id));

	// remaps elementid to elementname for 
	tracker_loadelementsused($tracker, $used);
	if (!empty($attributes)){
    	foreach(array_keys($attributes) as $attkey){
    	    $attributes[$attkey]->elementname = @$used[$attributes[$attkey]->id]->name;
    	}
    }
    $issue->attributes = $attributes;

    // We get comments and make a single backtrack. There should not 
    // be usefull to bring along full user profile. We just want not
    // to loose usefull information the previous track collected.
	$comments = $DB->get_records('tracker_issuecomment', array('issueid' => $issue->id));
    $track = '';
	if (!empty($comments)){
	    // collect userids
	    foreach($comments as $comment){
	        $useridsarray[] = $comment->userid;
	    }	    
	    $idlist = implode("','", $useridsarray);
	    $users = $DB->get_records_select('user', "id IN ('$idlist')", '', 'id, firstname, lastname');

        // make backtrack
        foreach($comments as $comment){
            $track .= get_string('commentedby', 'tracker').fullname($users[$comment->userid]).get_string('on', 'tracker').userdate($comment->datecreated);
            $track .= '<br/>';
            $track .= format_text($comment->comment, $comment->commentformat);
            $track .= '<hr width="60%"/>';
        }
	}
	$issue->comment = $track;
	
	// insert a backlink header in the content
	$olddescription = $issue->description;
	$issue->description = tracker_add_cascade_backlink($cm, $issue) . $issue->description;

    include_once $CFG->libdir."/pear/HTML/AJAX/JSON.php";
    include_once($CFG->dirroot.'/mod/tracker/rpclib.php');

    if (is_numeric($tracker->parent)){
        // tracker is local, use the rpc entry point anyway
        // emulate response
    	$result = tracker_rpc_post_issue($USER->username, $CFG->wwwroot, $tracker->parent, json_encode($issue));
        $rpcclient->response = get_object_vars($result);
    } else {
        // tracker is remote, make an RPC call

        list($remoteid, $mnet_host) = explode('@', $tracker->parent);

        // get network tracker properties
        include_once $CFG->dirroot.'/mnet/xmlrpc/client.php';
        $rpcclient = new mnet_xmlrpc_client();
        $rpcclient->set_method('mod/tracker/rpclib.php/tracker_rpc_post_issue');
        $rpcclient->add_param($USER->username, 'string');
        $rpcclient->add_param($CFG->wwwroot, 'string');
        $rpcclient->add_param($remoteid, 'int');
        $rpcclient->add_param(json_encode($issue), 'string');
        
        $parent_mnet = new mnet_peer();
        $parent_mnet->set_wwwroot($mnet_host);
        
        $result = $rpcclient->send($parent_mnet);               
    }
    if ($result){
        if ($rpcclient->response['status'] == RPC_SUCCESS){
            $issue->status = TRANSFERED;
            $issue->followid = $rpcclient->response['followid'];
            if (!$DB->update_record('tracker_issue', $issue)){
                print_error('erroroncacade', 'tracker');
            }
        } else {
            print_error('remotetrackererror', 'tracker', $result->error);            
        }
    } else {
        print_error ("Error on sending cascade :<br/>".implode('<br/>', $rpcclient->error));
    }
}
/********************************* raises the priority of the issue **************************/
elseif ($action == 'raisepriority'){
    $issueid = required_param('issueid', PARAM_INT);
    $issue = $DB->get_record('tracker_issue', array('id' => $issueid));
    $nextissue = $DB->get_record('tracker_issue', array('trackerid' => $tracker->id, 'resolutionpriority' => $issue->resolutionpriority + 1));
    if ($nextissue){
        $issue->resolutionpriority++;
        $nextissue->resolutionpriority--;
        $DB->update_record('tracker_issue', addslashes_recursive($issue));
        $DB->update_record('tracker_issue', addslashes_recursive($nextissue));
    }
}
/********************************* raises the priority at top of list **************************/
elseif ($action == 'raisetotop'){
    $issueid = required_param('issueid', PARAM_INT);
    $issue = $DB->get_record('tracker_issue', array('id' => $issueid));
    $maxpriority = $DB->get_field('tracker_issue', 'resolutionpriority', array('id' => $issueid));

    if ($issue->resolutionpriority != $maxpriority){
        // lower everyone above
        $sql = "
            UPDATE 
                {tracker_issue}
            SET 
                resolutionpriority = resolutionpriority - 1
            WHERE
                trackerid = ? AND
                resolutionpriority > ?
        ";
        execute_sql($sql, array($tracker->id, $issue->resolutionpriority));
    
        // update to max priority
        $issue->resolutionpriority = $maxpriority;
        $DB->update_record('tracker_issue', addslashes_recursive($issue));
    }
}
/********************************* lowers the priority of the issue **************************/
elseif ($action == 'lowerpriority'){
    $issueid = required_param('issueid', PARAM_INT);
    $issue = $DB->get_record('tracker_issue', array('id' => $issueid));
    if ($issue->resolutionpriority > 0){
        $nextissue = $DB->get_record('tracker_issue', array('trackerid' => $tracker->id, 'resolutionpriority' => $issue->resolutionpriority - 1));
        $issue->resolutionpriority--;
        $nextissue->resolutionpriority++;
        $DB->update_record('tracker_issue', addslashes_recursive($issue));
        $DB->update_record('tracker_issue', addslashes_recursive($nextissue));
    }
}
/********************************* raises the priority at top of list **************************/
elseif ($action == 'lowertobottom'){
    $issueid = required_param('issueid', PARAM_INT);
    $issue = $DB->get_record('tracker_issue', array('id' => $issueid));

    if ($issue->resolutionpriority > 0){
        // raise everyone beneath
        $sql = "
            UPDATE 
                {tracker_issue}
            SET 
                resolutionpriority = resolutionpriority + 1
            WHERE
                trackerid = ? AND
                resolutionpriority < ?
        ";
        execute_sql($sql, array($tracker->id, $issue->resolutionpriority));
    
        // update to min priority
        $issue->resolutionpriority = 0;
        $DB->update_record('tracker_issue', addslashes_recursive($issue));
    }
}
/****************** get some context for sending raising request ******************/
elseif ($action == 'askraise'){
     $issueid = required_param('issueid', PARAM_INT);

     include $CFG->dirroot.'/mod/tracker/views/raiserequest.html';
     
     return -1;
}
/****************** get some context for sending raising request ******************/
elseif ($action == 'doaskraise'){
     $issueid = required_param('issueid', PARAM_INT);
     $reason = required_param('reason', PARAM_TEXT);
     $urgent = required_param('urgent', PARAM_INT);
     
     $issue = $DB->get_record('tracker_issue', array('id' => $issueid));

     tracker_notify_raiserequest($issue, $cm, $reason, $urgent, $tracker);
}

?>